/*
 * Copyright (c) 2023 DTB <trinity@trinity.moe>
 * Copyright (c) 2024 Emma Tebibyte <emma@tebibyte.media>
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

#include <errno.h> /* errno */
#include <stdio.h> /* fprintf(3), stderr */
#include <stdlib.h> /* strtol(3), size_t, EXIT_FAILURE */
#include <unistd.h> /* getopt(3), optind */
#include <sysexits.h> /* EX_OK, EX_USAGE */

/* 0b00? */		/*         Equal                       | -e  | 0b001 | 1 */
#define EQUAL	0x01	/* Greater                     | -g  | 0b010 | 2 */
/* 0b0?0 */		/*         Greater or Equal            | -ge | 0b011 | 3 */
#define GREATER	0x02	/* Lesser                      | -l  | 0b100 | 4 */
/* 0b?00 */		/*         Lesser or Equal             | -le | 0b101 | 5 */
#define LESSER	0x04	/* Inequal (Greater or Lesser) | -gl | 0b110 | 6 */

static char *program_name = "intcmp";

int usage(char *s) {
	fprintf(stderr, "Usage: %s [-egl] integer integer...\n", s);

	return EX_USAGE;
}

int main(int argc, char *argv[]) {
	int c;
	size_t i;
	unsigned char mode;
	int r; /* reference integer */
	char *s = (argv[0] == NULL ? program_name : argv[0]);

	mode = 0;

	if (argc == 0 | argc < 3) { return usage(s); }

	while ((c = getopt(argc, argv, "egl")) != -1) {
		switch (c){
			case 'e': mode |= EQUAL;   break;
			case 'g': mode |= GREATER; break;
			case 'l': mode |= LESSER;  break;
			default:  return usage(s);
		}
	}

	if (optind + 2 /* ref cmp */ > argc) { return usage(s); }

	i = optind;

	do {
		r = c;
		c = strtol(argv[i], &argv[i], 10);

		if (*argv[i] != '\0' || errno != 0) {
			fprintf(
				stderr, "%s: argument #%d: Invalid integer\n", argv[0], (int)i
			);
			return EX_USAGE;
		}

		if (i == optind) { continue; }

		/* rule enforcement; if a mode isn't permitted and the numbers
		 * correspond to it, return 1 */
		if (   (!(mode & EQUAL) && r == c)
			|| (!(mode & GREATER) && r > c)
			|| (!(mode & LESSER) && r < c)
		) { return 1; }
	} while (++i < argc);
	
	return EX_OK;
}
